;;; DO NOT MODIFY THIS FILE
(if (featurep 'mule-base-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "mule-base/_pkg.el")

(package-provide 'mule-base :version 1.53 :author-version "No-Upstream-Ver" :type 'regular)

;;;***

;;;### (autoloads (view-charset) "char-table" "mule-base/char-table.el")

(autoload 'view-charset "char-table" "\
Display character table of CHARSET." t nil)

;;;***

;;;### (autoloads (view-charset-by-menu) "chartblxmas" "mule-base/chartblxmas.el")

(autoload 'view-charset-by-menu "chartblxmas" "\
Display character table of CHARSET by pop-up menu." t nil)

;;;***

;;;### (autoloads (encode-hz-buffer encode-hz-region decode-hz-buffer decode-hz-region setup-chinese-big5-environment setup-chinese-gb-environment setup-chinese-gb-environment-internal) "china-util" "mule-base/china-util.el")

(autoload 'setup-chinese-gb-environment-internal "china-util" nil nil nil)

(autoload 'setup-chinese-gb-environment "china-util" "\
Setup multilingual environment (MULE) for Chinese GB2312 users." t nil)

(autoload 'setup-chinese-big5-environment "china-util" "\
Setup multilingual environment (MULE) for Chinese Big5 users." t nil)

(autoload 'decode-hz-region "china-util" "\
Decode HZ/ZW encoded text in the current region.
Return the length of resulting text." t nil)

(autoload 'decode-hz-buffer "china-util" "\
Decode HZ/ZW encoded text in the current buffer." t nil)

(autoload 'encode-hz-region "china-util" "\
Encode the text in the current region to HZ.
Return the length of resulting text." t nil)

(autoload 'encode-hz-buffer "china-util" "\
Encode the text in the current buffer to HZ." t nil)

;;;***

;;;### (autoloads (standard-display-cyrillic-translit) "cyril-util" "mule-base/cyril-util.el")

(autoload 'standard-display-cyrillic-translit "cyril-util" "\
Display a cyrillic buffer using a transliteration.
For readability, the table is slightly
different from the one used for the input method `cyrillic-translit'.

The argument is a string which specifies which language you are using;
that affects the choice of transliterations slightly.
Possible values are listed in 'cyrillic-language-alist'.

Specifying a prefix arg, by preceding
\\[standard-display-cyrillic-translit] with \\[universal-argument]
turns off Cyrillic display.  Noninteractively, the DISABLE argument
does the same thing.  " t nil)

;;;***

;;;### (autoloads (ethio-write-file ethio-find-file ethio-java-to-fidel-buffer ethio-fidel-to-java-buffer ethio-tex-to-fidel-buffer ethio-fidel-to-tex-buffer ethio-input-special-character ethio-replace-space ethio-modify-vowel ethio-fidel-to-sera-marker ethio-fidel-to-sera-mail ethio-fidel-to-sera-mail-or-marker ethio-fidel-to-sera-buffer ethio-fidel-to-sera-region ethio-sera-to-fidel-marker ethio-sera-to-fidel-mail ethio-sera-to-fidel-mail-or-marker ethio-sera-to-fidel-buffer ethio-sera-to-fidel-region setup-ethiopic-environment-internal setup-ethiopic-environment) "ethio-util" "mule-base/ethio-util.el")

(autoload 'setup-ethiopic-environment "ethio-util" "\
Setup multilingual environment for Ethiopic." nil nil)

(autoload 'setup-ethiopic-environment-internal "ethio-util" nil nil nil)

(autoload 'ethio-sera-to-fidel-region "ethio-util" "\
Convert the characters in region from SERA to FIDEL.
The variable `ethio-primary-language' specifies the primary language
and `ethio-secondary-language' specifies the secondary.

If the 3rd parameter SECONDARY is given and non-nil, assume the region
begins begins with the secondary language; otherwise with the primary
language.

If the 4th parameter FORCE is given and non-nil, perform conversion
even if the buffer is read-only.

See also the descriptions of the variables
`ethio-use-colon-for-colon' and
`ethio-use-three-dot-question'." t nil)

(autoload 'ethio-sera-to-fidel-buffer "ethio-util" "\
Convert the current buffer from SERA to FIDEL.

The variable `ethio-primary-language' specifies the primary
language and `ethio-secondary-language' specifies the secondary.

If the 1st optional parameter SECONDARY is non-nil, assume the buffer
begins with the secondary language; otherwise with the primary
language.

If the 2nd optional parametr FORCE is non-nil, perform conversion even if the
buffer is read-only.

See also the descriptions of the variables
`ethio-use-colon-for-colon' and
`ethio-use-three-dot-question'." t nil)

(autoload 'ethio-sera-to-fidel-mail-or-marker "ethio-util" "\
Execute ethio-sera-to-fidel-mail or ethio-sera-to-fidel-marker depending on the current major mode.
If in rmail-mode or in mail-mode, execute the former; otherwise latter." t nil)

(autoload 'ethio-sera-to-fidel-mail "ethio-util" "\
Convert SERA to FIDEL to read/write mail and news.

If the buffer contains the markers \"<sera>\" and \"</sera>\",
convert the segments between them into FIDEL.

If invoked interactively and there is no marker, convert the subject field
and the body into FIDEL using `ethio-sera-to-fidel-region'." t nil)

(autoload 'ethio-sera-to-fidel-marker "ethio-util" "\
Convert the regions surrounded by \"<sera>\" and \"</sera>\" from SERA to FIDEL.
Assume that each region begins with `ethio-primary-language'.
The markers \"<sera>\" and \"</sera>\" themselves are not deleted." t nil)

(autoload 'ethio-fidel-to-sera-region "ethio-util" "\
Replace all the FIDEL characters in the region to the SERA format.
The variable `ethio-primary-language' specifies the primary
language and `ethio-secondary-language' specifies the secondary.

If the 3dr parameter SECONDARY is given and non-nil, try to convert
the region so that it begins in the secondary language; otherwise with
the primary language.

If the 4th parameter FORCE is given and non-nil, convert even if the
buffer is read-only.

See also the descriptions of the variables
`ethio-use-colon-for-colon', `ethio-use-three-dot-question',
`ethio-quote-vowel-always' and `ethio-numeric-reduction'." t nil)

(autoload 'ethio-fidel-to-sera-buffer "ethio-util" "\
Replace all the FIDEL characters in the current buffer to the SERA format.
The variable `ethio-primary-language' specifies the primary
language and `ethio-secondary-language' specifies the secondary.

If the 1st optional parameter SECONDARY is non-nil, try to convert the
region so that it begins in the secondary language; otherwise with the
primary language.

If the 2nd optional parameter FORCE is non-nil, convert even if the
buffer is read-only.

See also the descriptions of the variables
`ethio-use-colon-for-colon', `ethio-use-three-dot-question',
`ethio-quote-vowel-always' and `ethio-numeric-reduction'." t nil)

(autoload 'ethio-fidel-to-sera-mail-or-marker "ethio-util" "\
Execute ethio-fidel-to-sera-mail or ethio-fidel-to-sera-marker depending on the current major mode.
If in rmail-mode or in mail-mode, execute the former; otherwise latter." t nil)

(autoload 'ethio-fidel-to-sera-mail "ethio-util" "\
Convert FIDEL to SERA to read/write mail and news.

If the body contains at least one Ethiopic character,
 1) insert the string \"<sera>\" at the beginning of the body,
 2) insert \"</sera>\" at the end of the body, and
 3) convert the body into SERA.

The very same procedure applies to the subject field, too." t nil)

(autoload 'ethio-fidel-to-sera-marker "ethio-util" "\
Convert the regions surrounded by \"<sera>\" and \"</sera>\" from FIDEL to SERA.
The markers \"<sera>\" and \"</sera>\" themselves are not deleted." t nil)

(autoload 'ethio-modify-vowel "ethio-util" "\
Modify the vowel of the FIDEL that is under the cursor." t nil)

(autoload 'ethio-replace-space "ethio-util" "\
Replace ASCII spaces with Ethiopic word separators in the region.

In the specified region, replace word separators surrounded by two
Ethiopic characters, depending on the first parameter CH, which should
be 1, 2, or 3.

If CH = 1, word separator will be replaced with an ASCII space.
If CH = 2, with two ASCII spaces.
If CH = 3, with the Ethiopic colon-like word separator.

The second and third parameters BEGIN and END specify the region." t nil)

(autoload 'ethio-input-special-character "ethio-util" "\
Allow the user to input special characters." t nil)

(autoload 'ethio-fidel-to-tex-buffer "ethio-util" "\
Convert each fidel characters in the current buffer into a fidel-tex command.
Each command is always surrounded by braces." t nil)

(autoload 'ethio-tex-to-fidel-buffer "ethio-util" "\
Convert fidel-tex commands in the current buffer into fidel chars." t nil)

(autoload 'ethio-fidel-to-java-buffer "ethio-util" "\
Convert Ethiopic characters into the Java escape sequences.

Each escape sequence is of the form \\uXXXX, where XXXX is the
character's codepoint (in hex) in Unicode.

If `ethio-java-save-lowercase' is non-nil, use [0-9a-f].
Otherwise, [0-9A-F]." nil nil)

(autoload 'ethio-java-to-fidel-buffer "ethio-util" "\
Convert the Java escape sequences into corresponding Ethiopic characters." nil nil)

(autoload 'ethio-find-file "ethio-util" "\
Transcribe file content into Ethiopic dependig on filename suffix." nil nil)

(autoload 'ethio-write-file "ethio-util" "\
Transcribe Ethiopic characters in ASCII depending on the file extension." nil nil)

;;;***

;;;### (autoloads (isearch-fep-quail isearch-fep-canna isearch-fep-egg isearch-fep-string) "isearch-mule" "mule-base/isearch-mule.el")

(defvar search-string-char-prompt "*Enter string... ")

(autoload 'isearch-fep-string "isearch-mule" "\
Read string from minibuffer for incremental search." t nil)

(autoload 'isearch-fep-egg "isearch-mule" "\
Read string for incremental search by using egg." t nil)

(autoload 'isearch-fep-canna "isearch-mule" "\
Read string for incremental search by using canna." t nil)

(autoload 'isearch-fep-quail "isearch-mule" "\
Read string for incremental search by using quail." t nil)

;;;***

;;;### (autoloads (read-hiragana-string japanese-zenkaku-region japanese-hankaku-region japanese-hiragana-region japanese-katakana-region japanese-zenkaku japanese-hankaku japanese-hiragana japanese-katakana setup-japanese-environment-internal setup-japanese-environment) "japan-util" "mule-base/japan-util.el")

(autoload 'setup-japanese-environment "japan-util" "\
Setup multilingual environment (MULE) for Japanese." t nil)

(autoload 'setup-japanese-environment-internal "japan-util" nil nil nil)

(autoload 'japanese-katakana "japan-util" "\
Convert argument to Katakana and return that.
The argument may be a character or string.  The result has the same type.
The argument object is not altered--the value is a copy.
Optional argument HANKAKU t means to convert to `hankaku' Katakana
 (`japanese-jisx0201-kana'), in which case return value
 may be a string even if OBJ is a character if two Katakanas are
 necessary to represent OBJ." nil nil)

(autoload 'japanese-hiragana "japan-util" "\
Convert argument to Hiragana and return that.
The argument may be a character or string.  The result has the same type.
The argument object is not altered--the value is a copy." nil nil)

(autoload 'japanese-hankaku "japan-util" "\
Convert argument to `hankaku' and return that.
The argument may be a character or string.  The result has the same type.
The argument object is not altered--the value is a copy.
Optional argument ASCII-ONLY non-nil means to return only ASCII character." nil nil)

(autoload 'japanese-zenkaku "japan-util" "\
Convert argument to `zenkaku' and return that.
The argument may be a character or string.  The result has the same type.
The argument object is not altered--the value is a copy." nil nil)

(autoload 'japanese-katakana-region "japan-util" "\
Convert Japanese `hiragana' chars in the region to `katakana' chars.
Optional argument HANKAKU t means to convert to `hankaku katakana' character
of which charset is `japanese-jisx0201-kana'." t nil)

(autoload 'japanese-hiragana-region "japan-util" "\
Convert Japanese `katakana' chars in the region to `hiragana' chars." t nil)

(autoload 'japanese-hankaku-region "japan-util" "\
Convert Japanese `zenkaku' chars in the region to `hankaku' chars.
`Zenkaku' chars belong to `japanese-jisx0208'
`Hankaku' chars belong to `ascii' or `japanese-jisx0201-kana'.
Optional argument ASCII-ONLY non-nil means to convert only to ASCII char." t nil)

(autoload 'japanese-zenkaku-region "japan-util" "\
Convert hankaku' chars in the region to Japanese `zenkaku' chars.
`Zenkaku' chars belong to `japanese-jisx0208'
`Hankaku' chars belong to `ascii' or `japanese-jisx0201-kana'.
Optional argument KATAKANA-ONLY non-nil means to convert only KATAKANA char." t nil)

(autoload 'read-hiragana-string "japan-util" "\
Read a Hiragana string from the minibuffer, prompting with string PROMPT.
If non-nil, second arg INITIAL-INPUT is a string to insert before reading." nil nil)

;;;***

;;;### (autoloads (kana-keyboard-define-keys) "kana-keyboard" "mule-base/kana-keyboard.el")

(autoload 'kana-keyboard-define-keys "kana-keyboard" nil nil nil)

;;;***

;;;### (autoloads (setup-korean-environment-internal setup-korean-environment) "korea-util" "mule-base/korea-util.el")

(defvar default-korean-keyboard "" "\
*The kind of Korean keyboard for Korean input method.
\"\" for 2, \"3\" for 3.")

(autoload 'setup-korean-environment "korea-util" "\
Setup multilingual environment (MULE) for Korean." t nil)

(autoload 'setup-korean-environment-internal "korea-util" nil nil nil)

;;;***

;;;### (autoloads (dump-codings dump-charsets mule-diag list-input-methods list-fontsets describe-fontset describe-font list-coding-categories list-coding-systems describe-current-coding-system describe-current-coding-system-briefly describe-coding-system list-character-sets) "mule-diag" "mule-base/mule-diag.el")

(autoload 'list-character-sets "mule-diag" "\
Display a list of all character sets.

The ID column contains a charset identification number for internal use.
The B column contains a number of bytes occupied in a buffer
  by any character in this character set.
The W column contains a number of columns occupied on the screen
  by any character in this character set.

With prefix arg, the output format gets more cryptic,
but still shows the full information." t nil)

(autoload 'describe-coding-system "mule-diag" "\
Display information about CODING-SYSTEM." t nil)

(autoload 'describe-current-coding-system-briefly "mule-diag" "\
Display coding systems currently used in a brief format in echo area.

The format is \"F[..],K[..],T[..], default F[..]\",
where mnemonics of the following coding systems come in this order
at the place of `..':
  `buffer-file-coding-system` (of the current buffer)
  eol-type of buffer-file-coding-system (of the current buffer)
  Value returned by `keyboard-coding-system'
  eol-type of (keyboard-coding-system)
  Value returned by `terminal-coding-system.
  eol-type of (terminal-coding-system)
  `default-buffer-file-coding-system'
  eol-type of default-buffer-file-coding-system" t nil)

(autoload 'describe-current-coding-system "mule-diag" "\
Display coding systems currently used, in detail." t nil)

(autoload 'list-coding-systems "mule-diag" "\
Display a list of all coding systems.
This shows the mnemonic letter, name, and description of each coding system.

With prefix arg, the output format gets more cryptic,
but still contains full information about each coding system." t nil)

(autoload 'list-coding-categories "mule-diag" "\
Display a list of all coding categories." t nil)

(autoload 'describe-font "mule-diag" "\
Display information about fonts which partially match FONTNAME." t nil)

(autoload 'describe-fontset "mule-diag" "\
Display information of FONTSET.
This shows the name, size, and style of FONTSET, and the list of fonts
contained in FONTSET.

The column WDxHT contains width and height (pixels) of each fontset
\(i.e. those of ASCII font in the fontset).  The letter `-' in this
column means that the corresponding fontset is not yet used in any
frame.

The O column for each font contains one of the following letters:
 o -- font already opened
 - -- font not yet opened
 x -- font can't be opened
 ? -- no font specified

The Charset column for each font contains a name of character set
displayed (for this fontset) using that font." t nil)

(autoload 'list-fontsets "mule-diag" "\
Display a list of all fontsets.
This shows the name, size, and style of each fontset.
With prefix arg, it also list the fonts contained in each fontset;
see the function `describe-fontset' for the format of the list." t nil)

(autoload 'list-input-methods "mule-diag" "\
Display information about all input methods." t nil)

(autoload 'mule-diag "mule-diag" "\
Display diagnosis of the multilingual environment (Mule).

This shows various information related to the current multilingual
environment, including lists of input methods, coding systems,
character sets, and fontsets (if Emacs is running under a window
system which uses fontsets)." t nil)

(autoload 'dump-charsets "mule-diag" "\
Dump information about all charsets into the file `CHARSETS'.
The file is saved in the directory `data-directory'." nil nil)

(autoload 'dump-codings "mule-diag" "\
Dump information about all coding systems into the file `CODINGS'.
The file is saved in the directory `data-directory'." nil nil)

;;;***

;;;### (autoloads (set-keyboard-coding-system) "mule-keyboard" "mule-base/mule-keyboard.el")

(defvar keyboard-allow-latin-input nil "\
If non-nil, \"ESC , Fe\" and \"ESC - Fe\" are used for inputting
Latin characters.")

(autoload 'set-keyboard-coding-system "mule-keyboard" "\
Set variable keyboard-coding-system to CODESYS and modify keymap for it." t nil)

;;;***

;;;### (autoloads (define-word-regexp regexp-word-compile regexp-compile) "mule-trex" "mule-base/mule-trex.el")

(autoload 'regexp-compile "mule-trex" nil nil nil)

(autoload 'regexp-word-compile "mule-trex" nil nil nil)

(autoload 'define-word-regexp "mule-trex" nil nil 'macro)

;;;***

;;;### (autoloads (detect-coding-with-language-environment coding-system-eol-type-mnemonic lookup-nested-alist set-nested-alist nested-alist-p truncate-string-to-width store-substring) "mule-util" "mule-base/mule-util.el")

(autoload 'store-substring "mule-util" "\
Embed OBJ (string or character) at index IDX of STRING." nil nil)

(autoload 'truncate-string-to-width "mule-util" "\
Truncate string STR to end at column END-COLUMN.
The optional 2nd arg START-COLUMN, if non-nil, specifies
the starting column; that means to return the characters occupying
columns START-COLUMN ... END-COLUMN of STR.

The optional 3rd arg PADDING, if non-nil, specifies a padding character
to add at the end of the result if STR doesn't reach column END-COLUMN,
or if END-COLUMN comes in the middle of a character in STR.
PADDING is also added at the beginning of the result
if column START-COLUMN appears in the middle of a character in STR.

If PADDING is nil, no padding is added in these cases, so
the resulting string may be narrower than END-COLUMN." nil nil)

(defalias 'truncate-string 'truncate-string-to-width)

(autoload 'nested-alist-p "mule-util" "\
Return t if OBJ is a nested alist.

Nested alist is a list of the form (ENTRY . BRANCHES), where ENTRY is
any Lisp object, and BRANCHES is a list of cons cells of the form
\(KEY-ELEMENT . NESTED-ALIST).

You can use a nested alist to store any Lisp object (ENTRY) for a key
sequence KEYSEQ, where KEYSEQ is a sequence of KEY-ELEMENT.  KEYSEQ
can be a string, a vector, or a list." nil nil)

(autoload 'set-nested-alist "mule-util" "\
Set ENTRY for KEYSEQ in a nested alist ALIST.
Optional 4th arg LEN non-nil means the first LEN elements in KEYSEQ
 is considered.
Optional argument BRANCHES if non-nil is branches for a keyseq
longer than KEYSEQ.
See the documentation of `nested-alist-p' for more detail." nil nil)

(autoload 'lookup-nested-alist "mule-util" "\
Look up key sequence KEYSEQ in nested alist ALIST.  Return the definition.
Optional 1st argument LEN specifies the length of KEYSEQ.
Optional 2nd argument START specifies index of the starting key.
The returned value is normally a nested alist of which
car part is the entry for KEYSEQ.
If ALIST is not deep enough for KEYSEQ, return number which is
 how many key elements at the front of KEYSEQ it takes
 to reach a leaf in ALIST.
Optional 3rd argument NIL-FOR-TOO-LONG non-nil means return nil
 even if ALIST is not deep enough." nil nil)

(define-obsolete-function-alias 'set-coding-system-alist 'modify-coding-system-alist)

(autoload 'coding-system-eol-type-mnemonic "mule-util" "\
Return the string indicating end-of-line format of CODING-SYSTEM." nil nil)

(defsubst detect-coding-with-priority (from to priority-list) "Detect a coding system of the text between FROM and TO with PRIORITY-LIST.\nPRIORITY-LIST is an alist of coding categories vs the corresponding\ncoding systems ordered by priority." (let* ((orig-category-list (coding-priority-list)) (orig-category-systems (mapcar (function coding-category-system) orig-category-list))) (unwind-protect (let ((prio-list priority-list) categories x) (while prio-list (setq x (car prio-list)) (unless (memq (car x) categories) (set-coding-category-system (car x) (cdr x)) (setq categories (cons (car x) categories))) (setq prio-list (cdr prio-list))) (set-coding-priority-list (nreverse categories)) (detect-coding-region from to)) (set-coding-priority-list orig-category-list) (while orig-category-list (set-coding-category-system (car orig-category-list) (car orig-category-systems)) (setq orig-category-list (cdr orig-category-list) orig-category-systems (cdr orig-category-systems))))))

(autoload 'detect-coding-with-language-environment "mule-util" "\
Detect a coding system of the text between FROM and TO with LANG-ENV.
The detection takes into account the coding system priorities for the
language environment LANG-ENV." nil nil)

;;;***

;;;### (autoloads (thai-xtis-text-mode) "thai-xtis-util" "mule-base/thai-xtis-util.el")

(defvar thai-xtis-generic-char (if (featurep 'xemacs) 'thai-xtis (make-char 'thai-xtis)))

(autoload 'thai-xtis-text-mode "thai-xtis-util" "\
Minor mode for Thai text that pays attention to word segmentation.

In this mode, word-oriented commands (e.g forward-word) and text
filling commands (e.g. fill-paragraph)  recognize Thai word boundaries
within a sequence of Thai characters." t nil)

;;;***

;;;### (autoloads (viqr-pre-write-conversion viqr-post-read-conversion viet-encode-viqr-buffer viet-encode-viqr-region viet-decode-viqr-buffer viet-decode-viqr-region setup-vietnamese-environment viet-encode-viscii-char) "viet-util" "mule-base/viet-util.el")

(autoload 'viet-encode-viscii-char "viet-util" "\
Return VISCII character code of CHAR if appropriate." nil nil)

(autoload 'setup-vietnamese-environment "viet-util" "\
Setup multilingual environment (MULE) for Vietnamese VISCII users." t nil)

(autoload 'viet-decode-viqr-region "viet-util" "\
Convert `VIQR' mnemonics of the current region to Vietnamese characters.
When called from a program, expects two arguments,
positions (integers or markers) specifying the stretch of the region." t nil)

(autoload 'viet-decode-viqr-buffer "viet-util" "\
Convert `VIQR' mnemonics of the current buffer to Vietnamese characters." t nil)

(autoload 'viet-encode-viqr-region "viet-util" "\
Convert Vietnamese characters of the current region to `VIQR' mnemonics.
When called from a program, expects two arguments,
positions (integers or markers) specifying the stretch of the region." t nil)

(autoload 'viet-encode-viqr-buffer "viet-util" "\
Convert Vietnamese characters of the current buffer to `VIQR' mnemonics." t nil)

(autoload 'viqr-post-read-conversion "viet-util" nil nil nil)

(autoload 'viqr-pre-write-conversion "viet-util" nil nil nil)

;;;***

(provide 'mule-base-autoloads)
